import { HexBuffer } from '../HexBuffer';
import { W3Buffer } from '../W3Buffer';

enum ImportType {
    Standard,
    Custom
}

interface Import {
    path: string;
    type: ImportType;
}

export class ImportsTranslator {

    private _outBufferToWar: HexBuffer;
    private _outBufferToJSON: W3Buffer;

    constructor() { }

    public jsonToWar(imports: Import[]) {
        this._outBufferToWar = new HexBuffer();

        /*
         * Header
         */
        this._outBufferToWar.addInt(1); // file version
        this._outBufferToWar.addInt(imports.length); // number of imports

        /*
         * Body
         */
        imports.forEach((importedFile) => {
            this._outBufferToWar.addByte(
                importedFile.type === ImportType.Custom ? 13 : 8
            );

            // Temporary (disabled): always start the file path with war3mapImported\ until other file support is added
            // if (!importedFile.path.startsWith('war3mapImported\\') && importedFile.type === 'custom') {
            //     importedFile.path = 'war3mapImported\\' + importedFile.path;
            // }

            this._outBufferToWar.addString(importedFile.path, true);
        });

        return {
            errors: [],
            buffer: this._outBufferToWar.getBuffer()
        };
    }

    public warToJson(buffer: Buffer) {
        const result = [];
        this._outBufferToJSON = new W3Buffer(buffer);

        const fileVersion = this._outBufferToJSON.readInt(); // File version
        const numImports = this._outBufferToJSON.readInt(); // # of imports

        for (let i = 0; i < numImports; i++) {
            const typeValue = this._outBufferToJSON.readByte();

            // Standard: 0, 5, 8*
            // Custom: 10, 13*
            // * = World Editor uses this value
            const importType = [0, 5, 8].indexOf(typeValue) !== -1 ? ImportType.Standard : ImportType.Custom;

            const importedFile = {
                type: importType,
                path: this._outBufferToJSON.readString() // e.g. "war3mapImported\mysound.wav"
            };

            result.push(importedFile);
        }

        return {
            errors: [],
            json: result
        };
    }
}
