import { HexBuffer } from '../HexBuffer';

interface StringDefinition {
    id: number;
    text: string;
    comment?: string;
}

export class StringsTranslator {
    private _outBufferToWar: HexBuffer;

    constructor() { }

    public jsonToWar(stringsJson: StringDefinition[]) {
        this._outBufferToWar = new HexBuffer();

        // Add UTF8 byte-order mark (BOM), which is in the .wts file generated by World Editor
        // You can see the BOM when printing the buffer of the .wts file
        this._outBufferToWar.addBytes([0xef, 0xbb, 0xbf]); // see: https://en.wikipedia.org/wiki/Byte_order_mark#UTF-8

        /*
         * Strings
         */
        for (const strDef of stringsJson) {
            this._outBufferToWar.addString('STRING ' + strDef.id);
            this._outBufferToWar.addNewLine();

            if (strDef.comment) { // Comment can be empty, in which case skip this party
                this._outBufferToWar.addString(strDef.comment);
                this._outBufferToWar.addNewLine();
            }

            this._outBufferToWar.addString('{');
            this._outBufferToWar.addNewLine();

            this._outBufferToWar.addString(strDef.text);
            this._outBufferToWar.addNewLine();

            this._outBufferToWar.addString('}');
            this._outBufferToWar.addNewLine();

            this._outBufferToWar.addNewLine();
        }
        return {
            errors: [],
            buffer: this._outBufferToWar.getBuffer()
        };
    }

    public warToJson(buffer: Buffer) {
        const wts = buffer.toString().replace(/\r\n/g, '\n'), // may contain Windows linebreaks (\r\n), but below regex just assumes \n
            matchStringDefinitionBlock = new RegExp('STRING ([0-9]+)\n?((?:.*\n))?\{\n((?:.|\n)*?)\n}', 'g'); // see: https://regexr.com/4r6m0

        const result: StringDefinition[] = []; // stores the json form of strings file
        let match: RegExpExecArray; // stores individual matches as input is read

        // tslint:disable-next-line: no-conditional-assignment
        while ((match = matchStringDefinitionBlock.exec(wts)) !== null) {
            const num = parseInt(match[1], 10),
                comment = match[2]?.replace(/\n/gi, ''), // if the comment is present, the regex from above will have pulled out the newline, which needs to be stripped out
                body = match[3];

            result.push({
                id: num,
                text: body,
                comment
            });
        }

        return {
            errors: [],
            json: result
        };
    }
}
